using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Runtime.InteropServices;

namespace ScreenSaver2
{
    public partial class Form1 : Form
    {
        private string[] m_args;
        private Point textpos = new Point(0, 0);
        private Point v = new Point(1, 1);
        private Point MousePos;
        private Graphics g;
        private IntPtr m_hwnd;
        public struct RECT
        {
            public int Left;
            public int Top;
            public int Right;
            public int Bottom;
        }

        [DllImport("user32.dll")]
        static extern bool GetClientRect(IntPtr hWnd, out RECT lpRect);

        [DllImport("user32.dll", SetLastError = true)]
        static extern int GetWindowLong(IntPtr hWnd, int nIndex);

        // extra constants defined over and above the ones
        // defined in the article and used in the program
        const int GWL_ID = (-12);
        const int GWL_STYLE = (-16);
        const int GWL_EXSTYLE = (-20);
        const int WS_CHILD = 0x40000000;

        [DllImport("user32.dll")]
        static extern int SetWindowLong(IntPtr hWnd, int nIndex, int dwNewLong);

        [DllImport("user32.dll")]
        static extern IntPtr SetParent(IntPtr hWndChild, IntPtr hWndNewParent);
        const int GWL_HWNDPARENT = (-8);

        [DllImport("user32.dll")]
        static extern bool SetWindowPos(IntPtr hWnd, IntPtr hWndInsertAfter, int X,
           int Y, int cx, int cy, uint uFlags);

        // extra constants defined over and above the ones
        // defined in the article and used in the program
        
        static readonly IntPtr HWND_TOPMOST = new IntPtr(-1);
        static readonly IntPtr HWND_NOTOPMOST = new IntPtr(-2);
        static readonly IntPtr HWND_TOP = new IntPtr(0);
        static readonly IntPtr HWND_BOTTOM = new IntPtr(1);


        // extra constants defined over and above the ones
        // defined in the article and used in the program
        
        const UInt32 SWP_NOSIZE = 0x0001;
        const UInt32 SWP_NOMOVE = 0x0002;
        const UInt32 SWP_NOZORDER = 0x0004;
        const UInt32 SWP_NOREDRAW = 0x0008;
        const UInt32 SWP_NOACTIVATE = 0x0010;
        const UInt32 SWP_FRAMECHANGED = 0x0020;  /* The frame changed: send WM_NCCALCSIZE */
        const UInt32 SWP_SHOWWINDOW = 0x0040;
        const UInt32 SWP_HIDEWINDOW = 0x0080;
        const UInt32 SWP_NOCOPYBITS = 0x0100;
        const UInt32 SWP_NOOWNERZORDER = 0x0200;  /* Don't do owner Z ordering */
        const UInt32 SWP_NOSENDCHANGING = 0x0400;  /* Don't send WM_WINDOWPOSCHANGING */


        public Form1()
        {
            InitializeComponent();
            m_args = Environment.GetCommandLineArgs();
            this.TopLevel = false;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
 
            if (m_args.Length > 1)
            {
                string arg = m_args[1].ToLower().Trim().Substring(0, 2);
                switch (arg)
                {
                    case "/c":
                        //configure
                        break;
                    case "/s":

                        ShowScreenSaver();
                        break;
                    case "/p":
                        //preview
                        if (m_args.Length > 2)
                        {
                            this.TopLevel = false;
                            m_hwnd = (IntPtr)Int32.Parse(m_args[2]);
                            ShowScreenSaver();
                        }
                        break;
                }
            }
            else
            {
                ShowScreenSaver();
            }
        }


        void ShowScreenSaver()
        {

            if (this.m_hwnd == IntPtr.Zero)
            {
                this.FormBorderStyle = FormBorderStyle.None;
                this.TopLevel = true;
                this.WindowState = FormWindowState.Maximized;
                this.Capture = true;
                Cursor.Hide();
                this.DoubleBuffered = true;

            }
            else
            {
                RECT r = new RECT();
                this.FormBorderStyle = FormBorderStyle.None;
                GetClientRect(m_hwnd, out r);
                int style = GetWindowLong(this.Handle, GWL_STYLE);
                style = style | WS_CHILD;
                SetWindowLong(this.Handle, GWL_STYLE, style);
                SetParent(this.Handle, m_hwnd);
                SetWindowLong(this.Handle, GWL_HWNDPARENT, (int)m_hwnd);
                SetWindowPos(this.Handle, HWND_TOP, 0, 0, r.Right, r.Bottom,
                    SWP_NOZORDER | SWP_NOACTIVATE | SWP_SHOWWINDOW);

            }
            this.timer1.Interval = 50;
            this.timer1.Enabled = true;
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            g = this.CreateGraphics();
            RectangleF bounds = this.ClientRectangle;
            string displayM = "Visit I-Programmer";
            int fontsize = (int)bounds.Height / 20;
            FontFamily fontFamily = new FontFamily("Arial");
            Font font = new Font(
               fontFamily,
               fontsize,
               FontStyle.Regular,
               GraphicsUnit.Pixel);
            Size psize = new Size(int.MaxValue, int.MaxValue);
            Size size = TextRenderer.MeasureText(g, displayM, font, psize, TextFormatFlags.SingleLine);
            SolidBrush b = new SolidBrush(this.BackColor);
            g.FillRectangle(b, textpos.X, textpos.Y, size.Width, size.Height);
            textpos.X += v.X;
            textpos.Y += v.Y;
            if (textpos.X + size.Width >= bounds.Width || textpos.X <= 0) v.X = -v.X;
            if (textpos.Y + size.Height >= bounds.Height || textpos.Y < 0) v.Y = -v.Y;
            TextRenderer.DrawText(g, displayM, font, textpos, Color.Red);
            g.Dispose();

        }


        private void OnMouseEvent(object sender,
                          System.Windows.Forms.MouseEventArgs e)
        {
            if (!MousePos.IsEmpty)
            {
                if (MousePos != new Point(e.X, e.Y))
                {
                    g.Dispose();
                    Application.Exit();
                }
                if (e.Clicks > 0)
                {
                    g.Dispose();
                    Application.Exit();
                }
            }
            MousePos = new Point(e.X, e.Y);
        }
    }
}